# Intro to Git 

A introduction to ```git```, a program used to manage source code. When programming, it is very beneficial to be able to save versions of the code you are writing in a seamless way.

As code is the source to the programs we create, it is most efficient for us to store this 'source code' such that it can be built (turned into an executable program) when required, stored remotely (online) and easily shared.

> This work sheet depends on an understanding of the command line.
>
>  If you are unfamiliar with using the command line, then see the [CLI guide](https://learn-eu-central-1-prod-fleet01-xythos.content.blackboardcdn.com/5efb3ace8ea0a/11869645?X-Blackboard-Expiration=1633888800000&X-Blackboard-Signature=Va8Lr%2BFGLulgLS6OzwZeWi0aefGuqpIBp8QoC%2BncsNk%3D&X-Blackboard-Client-Id=147883&response-cache-control=private%2C%20max-age%3D21600&response-content-disposition=inline%3B%20filename%2A%3DUTF-8%27%27cli.pdf&response-content-type=application%2Fpdf&X-Amz-Algorithm=AWS4-HMAC-SHA256&X-Amz-Date=20211010T120000Z&X-Amz-SignedHeaders=host&X-Amz-Expires=21600&X-Amz-Credential=AKIAZH6WM4PL5M5HI5WH%2F20211010%2Feu-central-1%2Fs3%2Faws4_request&X-Amz-Signature=91149201ba9e27a35c8d3b7e77e595645524f4eb87133374bf919d230ef904e8).



Let's start with some basic git commands.



By passing git the argument ```version``` we can check the current version installed.

This is a good way to ensure we have git installed. If we get a 'git command not found' type of error, then look up how to install git on your platform somewhere such as [here](https://git-scm.com/book/en/v2/Getting-Started-Installing-Git).

```bash
git version
```



The git command acts on the current directory we are in. With a command such as ```git status``` we can check the status command however we will see that it is unlikely we are in a git repository. We would likely get an error such as:

```bash
git status

> fatal: not a git repository (or any of the parent directories): .git 
```



So lets get started working with git...

## Setting Up (First time only!)

If this is the first time you've used git on your machine then you will need to setup git to recognize your credentials.



```bash
git config --global user.name "FIRST_NAME LAST_NAME"
git config --global user.email "MY_NAME@example.com"
```



Be sure to use your UWE student email address in order to be compatible with the University online git repository.



## Git Repositories 

Git repositories or 'repos' as they are commonly referred to are projects that are being managed by git.



We can initialise a git repository in the directory we are in with the ```init``` command. So we should navigate create and/or navigate into a directory as required and initialise the repo like so:

```bash
mkdir someProject
cd someProject/
git init
> Initialized empty Git repository in /Users/name/someProject/.git/
```



We will then find, if we list the complete contents of this directory there is a new hidden directory in there called ```.git```. We **do not** need to work in this folder (which is why it is hidden), but it is good to understand that directory contains the inner workings of git. 



We can check the initialisation worked with the git ```status``` command. 

```sh
git status

> On branch master

> No commits yet

> nothing to commit (create/copy files and use "git add" to track)
```



As we can see by the output of the command, we are notified that we are on the master branch and that there are no commits to the repo yet.

> Note: we only need to initialise a repo if we are creating a new repo locally. If we are cloning an existing repo it is allready initialised

## Adding files

Now that we have a repository we will need to understand the basic work flow of git. 

#### Tracking files

If we wish to track files using git we must use the git ```add``` command to add the file to the repo.



We can add a newly created file like so, using the git status command to see the difference between before and after 'adding' the files:

```bash
touch example.txt
git status
git add example.txt
git status
```



Now that example.txt has been added, changes to this file will be tracked.

#### Saving: ```git commit```

To save a version of the tracked file (or files) we need to commit the changes we have made.

Every commit needs a message attached to it. We should use this message to indicate the changes we made for future reference. We can set the message using the ```-m``` command followed by a message surrounded by double quotes:

```bash
git commit -m "Added an empty example file."
```



>  Alternatively, if we just use the ```commit``` command on it's own, an editor will be opened to write the message. This will typically be the CLI editor ```vi``` or ```vim```. If you are unfamiliar with its usage, I would recommend escaping the editor by typing ```:q``` and using the method above.



## Remote Origins



Now that we have a repository with some files and a history of how those have changed we want to add a _remote origin_. This acts as a 'cloud backup'. We can __push__ and __pull__ changes from the remote origin to synchronize between local copies of the repository. 

At this point, I'm sure it's obvious what benefits just having a back up online of you code has so I won't go into details. But there are many more features in git that add to the obvious list such as collaboration and modifying different versions of the code. We won't go into those here though. 



To add a remote origin, we will use UWE's __GitLab__ which is a hosting platform for git repos that UWE can manage. You may have heard of others (Github being the most popular). 



In order to add a new remote origin, we should first go to [UWE's gitlab](https://gitlab.uwe.ac.uk) and login with your student credentials. (https://gitlab.uwe.ac.uk)



Create a new __empty__ repo by clicking the _new project button_.

Give it a name and optionally a description.



To add a remote origin to our repo we use the following command. *We need to remember to change the URL below to the one for our new repo though!*



```bash
git remote add origin https://gitlab.uwe.ac.uk/USER_NAME/PROJECT_NAME.git
```



We can find this by navigating to the repo and clicking the ```clone``` button and copying the **http url.**



![http url](git_resources/git_clone.png)



To synchronize you local repo with the new remote repo we need to use the following command:

```bash
git push -u origin master
```



## Finally

Ok, so now - armed with all that new knowledge and a git repo we can describe a workflow for continuing to use git when writing code...



Change Files 

**And/Or**

Add Files + Track with ```git add```



Commit files with ```git commit -m```



And then Push files to the remote repo with ```git push```



An example of this work flow in a terminal session may look like the following:

```bash
# create then add files required to git
# Notice we add multiple files at once with the git add command!
touch task1.cpp
touch task2.cpp
git add task1.cpp task2.cpp

# Code your solutions 
code .
# Probably alot of other stuff here...

# Save your solutions
git commit "There should often be multiple commits that describe what you did"

# Push your solution to the server
git push
```



From here you should go to the 'Git Submissions' worksheet to explore how we will submit code on this module.



For a more 'to the point' reference that is generally easier to find what you're looking for, check out the 'Git Reference' Worksheet!



# Resources



Ben's 3 part videos on source control (with git):

https://www.youtube.com/watch?v=25_mXw_veVg&list=PLe2QYG8u2CjMXsGSL7NOE4E9pe0Thd1gB



[blog: most basic git commands with examples](https://rubygarage.org/blog/most-basic-git-commands-with-examples.amp#referrer=https%3A%2F%2Fwww.google.com&amp_tf=From %1%24s)



[git explained in 100 seconds](https://www.youtube.com/watch?v=hwP7WQkmECE)



[How to use git and GitHub](https://www.youtube.com/watch?v=HkdAHXoRtos)

